#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

# __pragma__('skip')
from .middleware import Middleware
from .middleware_stack import MiddlewareStack
from .mutex import Mutex
from .logging_middleware import LoggingMiddleware
from .command_bus import CommandBus
from .event_bus import EventBus
from .query_bus import QueryBus
from .system_bus import SystemBus, SystemBusAware
from .service_executing_middleware import ServiceExecutingMiddleware
from .subscription_wrapper import SubscriptionWrapper
from .message_factory import MessageFactory
from .message_transport import MessageTransport
from .command_resolving_middleware import CommandResolvingMiddleware
from .event_resolving_middleware import EventResolvingMiddleware
from .query_resolving_middleware import QueryResolvingMiddleware
from .event_dispatching_middleware import EventDispatchingMiddleware
# __pragma__('noskip')
# __pragma__ ('ecom')
"""?
from firefly.domain.service.messaging.command_bus import CommandBus
from firefly.domain.service.messaging.event_bus import EventBus
from firefly.domain.service.messaging.message_factory import MessageFactory
from firefly.domain.service.messaging.middleware import Middleware
from firefly.domain.service.messaging.query_bus import QueryBus
from firefly.domain.service.messaging.system_bus import SystemBus
?"""
# __pragma__ ('noecom')



#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.
import os
import sys

from .cli import *
from .config import *
from .content_negotiation import *
from .core import *
from .files import *
from .http import *
from .logging import *
from .messaging import *
from .serialization import *


def set_env(func):
    for i in range(len(sys.argv)):
        if sys.argv[i] in ('--env', '-e'):
            os.environ['ENV'] = sys.argv[i + 1]
            return func
    os.environ['ENV'] = 'local'
    return func



#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Optional

from ...meta.meta_aware import MetaAware


class Handler(MetaAware, ABC):
    @abstractmethod
    def handle(self, *args, **kwargs) -> Optional[bool]:
        pass


class ChainOfResponsibility(ABC):
    def __init__(self):
        self._handlers = []

    def handle(self, *args, **kwargs):
        if len(self._handlers) == 0:
            return

        ret = False
        for handler in self._handlers:
            if handler.handle(*args, **kwargs) is True:
                ret = True
                break
        return ret

    def add(self, handler: Handler):
        self._handlers.append(handler)

    def remove(self, handler: Handler):
        self._handlers.remove(handler)

    @property
    def handlers(self):
        return self._handlers



#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

from __future__ import annotations

from typing import Generic, TypeVar

import firefly.domain as ffd

from .application_service import ApplicationService
from ...value_object.generic_base import GenericBase

T = TypeVar('T')


class QueryService(Generic[T], GenericBase, ApplicationService):
    _registry: ffd.Registry = None
    _serializer: ffd.Serializer = None

    def __call__(self, **kwargs):
        try:
            if self._type().id_name() in kwargs:
                return self._registry(self._type()).find(kwargs[self._type().id_name()])

            if 'criteria' in kwargs:
                criteria = kwargs['criteria']
                if isinstance(criteria, str):
                    criteria = self._serializer.deserialize(criteria)
                return list(self._registry(self._type()).filter(ffd.BinaryOp.from_dict(criteria)))
            else:
                return list(self._registry(self._type()))
        except KeyError:
            raise ffd.MissingArgument(self._type().id_name())



#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

from firefly.presentation.web.components.layouts.default import Crud
from firefly.presentation.web.polyfills import *  # __:skip


def add_route(route: str, vnode):
    if 'ff_routes' not in window:
        window.ff_routes = {}

    if isinstance(vnode, Crud):
        for k, v in vnode.routes().items():
            window.ff_routes[k] = v
        # window.ff_routes.update(vnode.routes())
    else:
        window.ff_routes[route] = vnode


def add_menu_item(vnode, index: int = None):
    if 'ff_menu' not in window:
        window.ff_menu = []
    if index is None:
        window.ff_menu.append(vnode)
    else:
        window.ff_menu.insert(index, vnode)



#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

import collections
import os
import gyp
import gyp.common
import gyp.msvs_emulation
import json
import sys

generator_supports_multiple_toolsets = True

generator_wants_static_library_dependencies_adjusted = False

generator_filelist_paths = {
}

generator_default_variables = {
}
for dirname in ['INTERMEDIATE_DIR', 'SHARED_INTERMEDIATE_DIR', 'PRODUCT_DIR',
                'LIB_DIR', 'SHARED_LIB_DIR']:
  # Some gyp steps fail if these are empty(!).
  generator_default_variables[dirname] = 'dir'
for unused in ['RULE_INPUT_PATH', 'RULE_INPUT_ROOT', 'RULE_INPUT_NAME',
               'RULE_INPUT_DIRNAME', 'RULE_INPUT_EXT',
               'EXECUTABLE_PREFIX', 'EXECUTABLE_SUFFIX',
               'STATIC_LIB_PREFIX', 'STATIC_LIB_SUFFIX',
               'SHARED_LIB_PREFIX', 'SHARED_LIB_SUFFIX',
               'CONFIGURATION_NAME']:
  generator_default_variables[unused] = ''


def CalculateVariables(default_variables, params):
  generator_flags = params.get('generator_flags', {})
  for key, val in generator_flags.items():
    default_variables.setdefault(key, val)
  default_variables.setdefault('OS', gyp.common.GetFlavor(params))

  flavor = gyp.common.GetFlavor(params)
  if flavor =='win':
    # Copy additional generator configuration data from VS, which is shared
    # by the Windows Ninja generator.
    import gyp.generator.msvs as msvs_generator
    generator_additional_non_configuration_keys = getattr(msvs_generator,
        'generator_additional_non_configuration_keys', [])
    generator_additional_path_sections = getattr(msvs_generator,
        'generator_additional_path_sections', [])

    gyp.msvs_emulation.CalculateCommonVariables(default_variables, params)


def CalculateGeneratorInputInfo(params):
  """Calculate the generator specific info that gets fed to input (called by
  gyp)."""
  generator_flags = params.get('generator_flags', {})
  if generator_flags.get('adjust_static_libraries', False):
    global generator_wants_static_library_dependencies_adjusted
    generator_wants_static_library_dependencies_adjusted = True

  toplevel = params['options'].toplevel_dir
  generator_dir = os.path.relpath(params['options'].generator_output or '.')
  # output_dir: relative path from generator_dir to the build directory.
  output_dir = generator_flags.get('output_dir', 'out')
  qualified_out_dir = os.path.normpath(os.path.join(
      toplevel, generator_dir, output_dir, 'gypfiles'))
  global generator_filelist_paths
  generator_filelist_paths = {
      'toplevel': toplevel,
      'qualified_out_dir': qualified_out_dir,
  }

def GenerateOutput(target_list, target_dicts, data, params):
  # Map of target -> list of targets it depends on.
  edges = {}

  # Queue of targets to visit.
  targets_to_visit = target_list[:]

  while len(targets_to_visit) > 0:
    target = targets_to_visit.pop()
    if target in edges:
      continue
    edges[target] = []

    for dep in target_dicts[target].get('dependencies', []):
      edges[target].append(dep)
      targets_to_visit.append(dep)

  try:
    filepath = params['generator_flags']['output_dir']
  except KeyError:
    filepath = '.'
  filename = os.path.join(filepath, 'dump.json')
  f = open(filename, 'w')
  json.dump(edges, f)
  f.close()
  print 'Wrote json to %s.' % filename



#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

from __future__ import annotations

import inspect
from typing import Union

import firefly as ff
import firefly.domain.error as error


class QueryHandler:
    def __call__(self, query: Union[str, type, None] = None):
        def query_wrapper(cls):
            try:
                cls.set_query(query)
            except AttributeError:
                if inspect.isfunction(cls):
                    ff.set_query(cls, query)
                else:
                    raise error.FrameworkError('@command_handler used on invalid target')
            return cls

        return query_wrapper


query_handler = QueryHandler()



#  Copyright (c) 2019 JD Williams
#
#  This file is part of Firefly, a Python SOA framework built by JD Williams. Firefly is free software; you can
#  redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 3 of the License, or (at your option) any later version.
#
#  Firefly is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
#  Public License for more details. You should have received a copy of the GNU Lesser General Public
#  License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#  You should have received a copy of the GNU General Public License along with Firefly. If not, see
#  <http://www.gnu.org/licenses/>.

import sys
import locale

reload(sys)

def main():
  encoding = locale.getdefaultlocale()[1]
  if not encoding:
    return False

  sys.setdefaultencoding(encoding)
  textmap = {
    'cp936': u'\u4e2d\u6587',
    'cp1252': u'Lat\u012Bna',
    'cp932': u'\u306b\u307b\u3093\u3054'
  }
  if textmap.has_key(encoding):
    print textmap[encoding]
  return True

if __name__ == '__main__':
  print main()
